#include <stdio.h>
#include <stdlib.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include "config.h"

#ifndef HAVE_GETOPT
#include "getopt.c"
#endif

char *getfield(char *line, int field);
char *urltouri(char *url);

#define BLOCKSIZE 1024

int main(int argc, char *argv[])
{
    int c;
    long int size = 0, slide = 0;
    char *line = NULL;
    char *outfile = NULL, *infile = NULL;
    int opt;
    FILE *fileout = NULL;
    FILE *filein = NULL;

    while ((opt = getopt(argc, argv, "i:o:rdhv")) != EOF) {
	switch (opt) {
	case 'i': {
	    infile = optarg;
	    break;
	}
	case 'o':
	    outfile = optarg;
	    break;

	case 'h':
	    printf("urltouri [-i input-file] [-o output-file] [url] ...\n");
	    exit(0);

	case 'v':
	    printf("urltouri - " PACKAGE " " VERSION "\n");
	    exit(0);
	}
    }

    /* open up file for output */
    if (outfile) {
	if (!(fileout = fopen(outfile, "w"))) {
	    perror(outfile);
	    exit(1);
	}
    }
    else
	fileout = stdout;

    /* open up and read input file */
    if (infile) {
	if (!(filein = fopen(infile, "r"))) {
	    perror(infile);
	    exit(1);
	}
    }
    else
	filein = stdin;




    /* either we convert URLs on the command line, or URLs in a file */
    opt = optind;
    if (argc > opt) {

	while (argc > opt) {

	    fprintf (fileout, "%s\n", urltouri(argv[opt]));
	    opt++;
	}   

    }
    else {

	/* get each line of text until EOF */
	while ((c = fgetc(filein)) != EOF) {
	    /* enough space in buffer? no, make some more */
	    if (slide >= size) {
		size += BLOCKSIZE;
		if (!(line = (char *) realloc(line, size))) {
		    fprintf(stderr, "Memory allocation error\n");
		    exit(1);
		}
	    }

	    /* save character to buffer */
	    line[slide++] = (char) c;

	    /* reached an end of line? if so, deal with the line */
	    if (c == 10) {

		/* the line may be terminated with a CR, if so chop it off
		 * otherwise replace the LF with a NULL to indicate end of string */
		if ((slide > 0) && (line[slide - 1] == 13))
		    line[slide - 2] = 0;
		else
		    line[slide - 1] = 0;

		/* print the line */
		fprintf (fileout, "%s\n", urltouri(line));



		/* clean up */
		free(line);
		line = NULL;
		slide = 0;
		size = 0;
	    }
	}
	if (line)
	    free(line);
    }


    /* close input file (but only if the file was a real file) */
    if (filein && filein != stdin)
	fclose(filein);

    /* close out output file */
    if (fileout != stdout)
	fclose(fileout);

    exit(0);
}



/* get field number in a line */
/* buffer returned is size + 127 bytes so that it may be overwritten by the calling routine */
char *getfield(char *line, int field)
{
    char *buf;
    int start = 0, end = 0, slide = 0;
    int ws = 0;
    int br = 0;

    /* if ws is 0, we are finding text. if ws is not 0, we are finding whitespace */
    /* if br is not 0, we are inside a set of brackets */
    while (field && line[slide] && (line[slide] != 13)) {

	/* deal with brackets */
	if (line[slide] == '"') {
	    if (br)
		br = 0;
	    else
		br = 1;
	}

	/* find whitespace */
	switch (line[slide]) {
	case 8:
	case ' ':{
		if (!br && ws) {
		    ws = 0;
		}
		break;
	    }
	default:{
		if (!ws) {
		    field--;
		    ws = 1;
		    start = slide;
		}
	    }
	}
	slide++;
    }

    /* we found the start, now find the end */
    while (!end && line[slide] && (line[slide] != 13)) {

	/* find whitespace */
	switch (line[slide]) {
	case '"':{
		if (br)
		    br = 0;
		else
		    br = 1;
		break;
	    }
	case 8:
	case ' ':{
		if (!br) {
		    end = slide;
		}
		break;
	    }
	}
	slide++;
    }

    /* if !end, we found the last field */
    if (!end)
	end = slide;

    /* if field not found, return "" */
    if (field)
	start = end = 0;

    /* create a buffer for the field and return it - buffer contains an extra 127 bytes */
    if (!(buf = (char *) malloc(end - start + 128)))
	return (NULL);
    strncpy(buf, line + start, end - start);
    buf[end - start] = 0;
    return (buf);
}

char *urltouri(char *url)
{
    long x = 0, s = 0;

    /* check if the URL starts with a known prefix */
    if ( !(strncmp(url, "http://", 7)) || !(strncmp(url, "https://", 8)) || !(strncmp(url, "ftp://", 6)) || !(strncmp(url, "ldap://", 7)) ) {

	/* look for the next / character or end of string */
	while(url[x]) {
	    if (url[x] == '/') {
		s++;
	    }
	    if (s == 3) break;
	    x++;
	}

	/* move the string, or don't. */
	if (s == 3) {
	    memmove(url, url+x, strlen(url)-x+1);
	}
	else {
	    strcpy(url, "/");
	}
    }
    return(url);

}
